/*============================================================================
  File:     ShrinkFragmentationDemo.sql

  Summary:  This script shows how data file
			shrink can cause massive index
			fragmentation

  Date:     June 2009

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE MASTER;
GO

IF DATABASEPROPERTYEX ('DBMaint2008', 'Version') > 0
	DROP DATABASE DBMaint2008;

CREATE DATABASE DBMaint2008;
GO
USE DBMaint2008;
GO

SET NOCOUNT ON;
GO

-- Create the 10MB filler table at the
-- 'front' of the data file
CREATE TABLE FillerTable (
	c1 INT IDENTITY,
	c2 CHAR (8000) DEFAULT 'filler');
GO

-- Fill up the filler table
INSERT INTO FillerTable DEFAULT VALUES;
GO 1280

-- Create the production table, which
-- will be 'after' the filler table in
-- the data file
CREATE TABLE ProdTable (
	c1 INT IDENTITY,
	c2 CHAR (8000) DEFAULT 'production');
CREATE CLUSTERED INDEX prod_cl
	ON ProdTable (c1);
GO

INSERT INTO ProdTable DEFAULT VALUES;
GO 1280

-- check the fragmentation of the production table
SELECT
	[avg_fragmentation_in_percent]
FROM sys.dm_db_index_physical_stats (
	DB_ID ('DBMaint2008'),
	OBJECT_ID ('ProdTable'),
	1,
	NULL,
	'LIMITED');
GO

-- drop the filler table, creating 10MB
-- of free space at the 'front' of the
-- data file
DROP TABLE FillerTable;
GO

-- shrink the database
DBCC SHRINKDATABASE (DBMaint2008);
GO

-- check the index fragmentation again
SELECT
	[avg_fragmentation_in_percent]
FROM sys.dm_db_index_physical_stats (
	DB_ID ('DBMaint2008'),
	OBJECT_ID ('ProdTable'),
	1,
	NULL,
	'LIMITED');
GO