/*============================================================================
  File:     CauseAndDetectFragmentation.sql

  Summary:  This script shows schemas that can
			lead to fragmentation and how to 
			detect fragmentation

  Date:     June 2009

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE MASTER;
GO

IF DATABASEPROPERTYEX ('DBMaint2008', 'Version') > 0
	DROP DATABASE DBMaint2008;

CREATE DATABASE DBMaint2008;
GO
USE DBMaint2008;
GO

SET NOCOUNT ON;
GO

-- Create a table with a GUID key
CREATE TABLE BadKeyTable (
	c1 UNIQUEIDENTIFIER DEFAULT NEWID () ROWGUIDCOL,
    c2 DATETIME DEFAULT GETDATE (),
	c3 CHAR (400) DEFAULT 'a',
	c4 VARCHAR(MAX) DEFAULT 'b');
CREATE CLUSTERED INDEX BadKeyTable_CL ON
	BadKeyTable (c1);
CREATE NONCLUSTERED INDEX BadKeyTable_NCL ON
	BadKeyTable (c2);
GO

-- Create another one, but using
-- NEWSEQUENTIALID instead
CREATE TABLE BadKeyTable2 (
	c1 UNIQUEIDENTIFIER DEFAULT NEWSEQUENTIALID () ROWGUIDCOL,
    c2 DATETIME DEFAULT GETDATE (),
	c3 CHAR (400) DEFAULT 'a',
	c4 VARCHAR(MAX) DEFAULT 'b');
CREATE CLUSTERED INDEX BadKeyTable2_CL ON
	BadKeyTable2 (c1);
CREATE NONCLUSTERED INDEX BadKeyTable2_NCL ON
	BadKeyTable2 (c2);
GO

DECLARE @a INT;
SELECT @a = 1;
WHILE (@a < 10000)
BEGIN
	INSERT INTO BadKeyTable DEFAULT VALUES;
	INSERT INTO BadKeyTable2 DEFAULT VALUES;
	SELECT @a = @a + 1;
END;
GO

-- Check the fragmentation of the indexes in
-- the database
SELECT * FROM sys.dm_db_index_physical_stats (
	DB_ID ('DBMaint2008'),
	NULL,
	NULL,
	NULL,
	'DETAILED');
GO

-- And now with the LIMITED option
SELECT * FROM sys.dm_db_index_physical_stats (
	DB_ID ('DBMaint2008'),
	NULL,
	NULL,
	NULL,
	'LIMITED');
GO

-- And now with a bit more useful info
SELECT
	OBJECT_NAME (ips.[object_id]) AS 'Object Name',
	si.name AS 'Index Name',
	ROUND (ips.avg_fragmentation_in_percent, 2) AS 'Fragmentation',
	ips.page_count AS 'Pages',
	ROUND (ips.avg_page_space_used_in_percent, 2) AS 'Page Density'
FROM sys.dm_db_index_physical_stats (
	DB_ID ('DBMaint2008'),
	NULL,
	NULL,
	NULL,
	'DETAILED') ips
CROSS APPLY sys.indexes si
WHERE
	si.object_id = ips.object_id
	AND si.index_id = ips.index_id
	AND ips.index_level = 0
GO

-- Now do the whole thing again using SMALLDATETIME
-- to show the different NC index fragmentation