/*============================================================================
  File:     RemoveFragmentation.sql

  Summary:  This script shows how to remove
			fragmentation

  Date:     June 2009

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE DBMaint2008;
GO

-- Look at the fragmentation again
SELECT
	OBJECT_NAME (ips.[object_id]) AS 'Object Name',
	si.name AS 'Index Name',
	ips.avg_fragmentation_in_percent AS 'Fragmentation',
	ips.page_count AS 'Pages',
	ips.avg_page_space_used_in_percent AS 'Page Density'
FROM sys.dm_db_index_physical_stats (
	DB_ID ('DBMaint2008'),
	NULL,
	NULL,
	NULL,
	'DETAILED') ips
CROSS APPLY sys.indexes si
WHERE
	si.object_id = ips.object_id
	AND si.index_id = ips.index_id
	AND ips.index_level = 0
GO

-- Online rebuild the clustered index
ALTER INDEX BadKeyTable_CL
ON BadKeyTable REBUILD
WITH (ONLINE = ON, FILLFACTOR = 70);
GO

-- Ok - offline then
ALTER INDEX BadKeyTable_CL
ON BadKeyTable REBUILD
WITH (FILLFACTOR = 70);
GO

-- Reorganize the non-clustered index
ALTER INDEX BadKeyTable_NCL
ON BadKeyTable REORGANIZE;
GO

-- And check again...

-- Now rebuild the non-clustered index
ALTER INDEX BadKeyTable_NCL
ON BadKeyTable REBUILD
WITH (ONLINE = ON, FILLFACTOR = 70);
GO

-- And check again...